'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var OAuth = require('./oauth');
var qs = require('querystring');
var events = require('events');
var util = require('util');
var request = require('request');
var oauthSignature = require('oauth-signature');
var Timeline = require('./timeline');
var Status = require('./status');
var Streaming = require('./streaming');

var Fanfou = function () {
  function Fanfou(options) {
    _classCallCheck(this, Fanfou);

    options = options || {};

    // required
    this.consumer_key = options.consumer_key;
    this.consumer_secret = options.consumer_secret;
    this.auth_type = options.auth_type;

    // optional
    this.protocol = options.protocol || 'http:';
    this.api_domain = options.api_domain || 'api.fanfou.com';
    this.streaming_domain = options.streaming_domain || 'stream.fanfou.com';

    // oauth required
    if (this.auth_type === 'oauth') {
      this.oauth_token = options.oauth_token || '';
      this.oauth_token_secret = options.oauth_token_secret || '';
    }

    // xauth required
    if (this.auth_type === 'xauth') {
      this.username = options.username || '';
      this.password = options.password || '';
    }

    this.is_streaming = false;
    this.oauth = new OAuth('http://fanfou.com/oauth/request_token', 'http://fanfou.com/oauth/access_token', this.consumer_key, this.consumer_secret, '1.0', null, 'HMAC-SHA1');
  }

  _createClass(Fanfou, [{
    key: 'xauth',
    value: function xauth(callback) {
      var _this = this;

      this.oauth.getXAuthAccessToken(this.username, this.password, function (e, oauth_token, oauth_token_secret, result) {
        if (e) callback(e);else {
          _this.oauth.oauth_token = oauth_token;
          _this.oauth.oauth_token_secret = oauth_token_secret;
          callback(null, {
            oauth_token: oauth_token,
            oauth_token_secret: oauth_token_secret
          });
        }
      });
    }

    /**
     * @param uri
     * @param parameters
     * @param callback
     */

  }, {
    key: 'get',
    value: function get(uri, parameters, callback) {
      var url = this.protocol + '//' + this.api_domain + uri + '.json';
      this.oauth.get(url + '?' + qs.stringify(parameters), this.oauth_token, this.oauth_token_secret, function (e, data, res) {
        // TODO http status code
        if (e) callback(e, null, null);else {
          if (Fanfou._uriType(uri) === 'timeline') {
            callback(null, data, new Timeline(JSON.parse(data)));
          } else if (Fanfou._uriType(uri) === 'status') {
            callback(null, data, new Status(JSON.parse(data)));
          } else {
            callback(null, data, null);
          }
        }
      });
    }

    /**
     * @param uri
     * @param parameters
     * @param callback
     */

  }, {
    key: 'post',
    value: function post(uri, parameters, callback) {
      var url = this.protocol + '//' + this.api_domain + uri + '.json';
      this.oauth.post(url, this.oauth_token, this.oauth_token_secret, parameters, function (e, data, res) {
        if (e) callback(e, null);else {
          if (Fanfou._uriType(uri) === 'timeline') {
            callback(null, data, new Timeline(JSON.parse(data)));
          } else if (Fanfou._uriType(uri) === 'status') {
            callback(null, data, new Status(JSON.parse(data)));
          } else {
            callback(null, data, null);
          }
        }
      });
    }

    /**
     * @param uri
     * @param parameters
     */

  }, {
    key: 'stream',
    value: function stream(uri, parameters) {
      var _this2 = this;

      // prevent concurrences
      if (this.is_streaming === true) {
        console.warn('A previous streamer of this instance is currently running, cannot create more.');
        return;
      }

      // params validation
      if (uri === undefined) {
        uri = '/user';
      }

      if ((typeof parameters === 'undefined' ? 'undefined' : _typeof(parameters)) !== 'object') {
        parameters = {};
      }

      var url = this.protocol + '//' + this.streaming_domain + '/1' + uri + '.json';
      var request = this.oauth.post(url, this.oauth_token, this.oauth_token_secret, parameters, null);

      var ee = new events.EventEmitter();

      ee.stop = function () {
        request.abort();
        _this2.is_streaming = false;
      };

      request.on('error', function (error) {
        ee.emit('error', { type: 'request', data: error });
        _this2.is_streaming = false;
      });

      // init and subsequent response data
      request.on('response', function (response) {
        if (response.statusCode > 200) {
          ee.emit('error', { type: 'response', data: { code: response.statusCode } });
          _this2.is_streaming = false;
        } else {
          _this2.is_streaming = true;

          ee.emit('connected');

          response.setEncoding('utf8');
          var data = '';

          response.on('data', function (chunk) {
            data += chunk.toString('utf8');

            if (data === '\r\n') {
              ee.emit('heartbeat');
              return;
            }

            var index = void 0,
                json = void 0;

            while ((index = data.indexOf('\r\n')) > -1) {
              json = data.slice(0, index);
              data = data.slice(index + 2);
              if (json.length > 0) {
                try {
                  var newStreaming = new Streaming(JSON.parse(json));
                  ee.emit(newStreaming.schema, newStreaming);
                } catch (e) {
                  ee.emit('garbage', data);
                }
              }
            }
          });

          response.on('error', function (error) {
            ee.emit('close', error);
          });

          response.on('end', function () {
            ee.emit('close', 'connection dropped');
          });

          response.on('close', function () {
            request.abort();
            _this2.is_streaming = false;
          });
        }
      });

      request.end();

      return ee;
    }

    /**
     * @param stream
     * @param text
     * @param callback
     */

  }, {
    key: 'upload',
    value: function upload(stream, text, callback) {
      var method = 'POST';
      var url = this.protocol + '//' + this.api_domain + '/photos/upload.json';
      var params = {
        oauth_consumer_key: this.consumer_key,
        oauth_token: this.oauth_token,
        oauth_signature_method: 'HMAC-SHA1',
        oauth_timestamp: Math.floor(Date.now() / 1000),
        oauth_nonce: this.oauth._getNonce(6),
        oauth_version: '1.0'
      };
      var signature = oauthSignature.generate(method, url, params, this.consumer_secret, this.oauth_token_secret, { encodeSignature: false });
      var authorizationHeader = this.oauth._buildAuthorizationHeaders(this.oauth._sortRequestParams(this.oauth._makeArrayOfArgumentsHash(params)).concat([['oauth_signature', signature]]));
      var formData = {
        photo: stream,
        status: text
      };
      request.post({
        url: url,
        formData: formData,
        headers: { Authorization: authorizationHeader }
      }, function (err, httpResponse, body) {
        if (err) callback(err, null, null);else if (httpResponse.statusCode !== 200) callback(body, null, null);else callback(null, body, new Status(JSON.parse(body)));
      });
    }

    /**
     * @param uri
     * @returns {string}
     * @private
     */

  }], [{
    key: '_uriType',
    value: function _uriType(uri) {
      var uriList = {
        '/search/public_timeline': 'timeline',
        '/search/user_timeline': 'timeline',
        '/photos/user_timeline': 'timeline',
        '/statuses/friends_timeine': 'timeline',
        '/statuses/home_timeline': 'timeline',
        '/statuses/public_timeline': 'timeline',
        '/statuses/replies': 'timeline',
        '/statuses/user_timeline': 'timeline',
        '/statuses/context_timeline': 'timeline',
        '/statuses/mentions': 'timeline',
        '/favorites': 'timeline',
        '/statuses/destroy': 'status',
        '/statuses/update': 'status',
        '/statuses/show': 'status',
        '/favorites/destroy': 'status',
        '/favorites/create': 'status'
      };
      return uriList[uri] || null;
    }
  }]);

  return Fanfou;
}();

module.exports = Fanfou;